<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:doc="http://www.docbook.org/xmlns/docbook" xmlns:xbl="http://www.mozilla.org/xbl"
    version="2.0" xmlns="http://www.w3.org/1999/xhtml">
    <xsl:output method="xml" media-type="text/html" indent="yes" cdata-section-elements="pre style"/>
    <xsl:template match="/">
        <xsl:apply-templates select="xbl:bindings"/>
    </xsl:template>
    <xsl:template match="xbl:bindings">
        <xsl:processing-instruction name="xml-stylesheet"> type="text/css" href="viewXBL.css"</xsl:processing-instruction>
        <html>
            <head>
                <title>Bindings</title>
            </head>
            <body>
                <div class="header">Mercurial Communications Documentation&#160;&#160;</div>
                <h1>Table of Contents</h1>
                <ul>
                    <xsl:apply-templates select="xbl:binding" mode="toc"/>
                </ul>
                <xsl:apply-templates select="xbl:binding"/>
            </body>
        </html>
    </xsl:template>
    <xsl:template match="xbl:binding" mode="toc">
        <li>
            <a href="#{@id}">
                <xsl:apply-templates select="." mode="title"/>
            </a>
        </li>
    </xsl:template>
    <xsl:template match="xbl:binding">
        <hr/>
        <h1>
            <a name="{@id}"/>
            <xsl:apply-templates select="." mode="title"/>
        </h1>
        <xsl:apply-templates select="doc:*"/>
        <hr/>
        <xsl:apply-templates select="xbl:implementation"/>
    </xsl:template>
    <xsl:template match="xbl:implementation">
        <div class="indent">
            <xsl:if test="xbl:field">
                <h2>Fields</h2>
            <table>
            <tr><th>Name</th><th>Description</th></tr>    
            <xsl:apply-templates select="xbl:field">
                <xsl:sort order="ascending" select="@name"/>
            </xsl:apply-templates>
            </table>
            </xsl:if>
            <xsl:if test="xbl:property">
                <h2>Properties</h2>
            <table>                
            <tr><th>Name</th><th>Setter</th><th>Getter</th><th>Description</th></tr>    
            <xsl:apply-templates select="xbl:property">
                <xsl:sort order="ascending" select="@name"/>
            </xsl:apply-templates>
            </table>
            </xsl:if>
            <xsl:if test="xbl:method">
                <h2>Methods</h2>
            <table>
            <tr><th>Name</th><th>Description</th></tr>    
            <xsl:apply-templates select="xbl:method">
                <xsl:sort order="ascending" select="@name"/>
            </xsl:apply-templates>
            </table>    
            </xsl:if>
        </div>
    </xsl:template>
    <xsl:template match="xbl:method">
        <tr>
            <td class="row_{position() mod 2}"><xsl:value-of select="@name"/>(<xsl:for-each select="xbl:parameter/@name"><xsl:value-of select="."/><xsl:if test="position()!=last()">,<xsl:text> </xsl:text></xsl:if></xsl:for-each>)</td>
            <td class="row_{position() mod 2}"><xsl:apply-templates select="doc:description" mode="table"/></td>
<!--            <xsl:apply-templates select="xbl:body"/>-->
        </tr>
    </xsl:template>
    <xsl:template match="xbl:property">
        <tr>
            <td  class="row_{position() mod 2}"><xsl:value-of select="@name"/>
            </td>
            <td class="row_{position() mod 2}">
                <xsl:choose>
                <xsl:when test="xbl:setter|@xbl:setter">Yes</xsl:when>
                <xsl:otherwise>No</xsl:otherwise>
                </xsl:choose>
            </td>
            <td class="row_{position() mod 2}">
                <xsl:choose>
                <xsl:when test="xbl:getter|@xbl:getter">Yes</xsl:when>
                <xsl:otherwise>No</xsl:otherwise>
                </xsl:choose>
            </td>
            <td class="row_{position() mod 2}"><xsl:apply-templates select="doc:description" mode="table"/></td>
            <!--<xsl:apply-templates select="*"/>-->
        </tr>
    </xsl:template>
    <xsl:template match="xbl:field">
        <tr>
            <td class="row_{position() mod 2}"><xsl:value-of select="@name"/>
            </td>
            <td class="row_{position() mod 2}"><xsl:apply-templates select="doc:description" mode="table"/></td>
        </tr>
    </xsl:template>
    <xsl:template match="xbl:body">
        <pre>
            <xsl:value-of select="."/>
        </pre>
    </xsl:template>
    <xsl:template match="xbl:property/text()">
        <pre>
            <xsl:value-of select="."/>
        </pre>
    </xsl:template>
    <xsl:template match="xbl:getter|@getter">
        <div class="indent">
            <h4>Getter</h4>
            <pre>
                <xsl:value-of select="."/>
            </pre>
        </div>
    </xsl:template>
    <xsl:template match="xbl:setter|@setter">
        <div class="indent">
            <h4>Setter</h4>
            <pre>
                <xsl:value-of select="."/>
            </pre>
        </div>
    </xsl:template>
    <xsl:template match="*|@*|text()">
        <xsl:copy>
            <xsl:apply-templates select="*|@*|text()"/>
        </xsl:copy>
    </xsl:template>
    <xsl:template match="doc:*">
        <div>
            <span class="metatitle">
                <xsl:value-of select="local-name(.)"/>: </span>
            <span class="metacontents">
                <xsl:value-of select="."/>
            </span>
        </div>
    </xsl:template>
    <xsl:template match="doc:title"/>
    <xsl:template match="doc:description">
        <div class="description">
            <xsl:apply-templates select="*|text()"/>
        </div>
    </xsl:template>
    <xsl:template match="doc:description" mode="table">
       <xsl:apply-templates select="*|text()"/>
    </xsl:template>
    <xsl:template match="doc:testbed">
        <div>
            <span class="metatitle">
                <xsl:value-of select="local-name(.)"/>: </span>
            <span class="metacontents">
               <a href="{.}"><xsl:apply-templates select="*|text()"/></a>
            </span>
        </div>
    </xsl:template>
    <xsl:template match="doc:element">
        <div>
            <span class="metatitle">
                <xsl:value-of select="local-name(.)"/>: </span>
            <span class="metacontents">
                &lt;<xsl:value-of select="."/>&gt;
            </span>
        </div>
    </xsl:template>
    <xsl:template match="doc:description" mode="table">
       <xsl:apply-templates select="*|text()"/>
    </xsl:template>
    
    <xsl:template match="*" mode="title">
        <xsl:choose>
            <xsl:when test="doc:title">
                <xsl:value-of select="doc:title"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="@id"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
</xsl:stylesheet>
